@@ -81,17 +81,29 @@ void MH_FormationTNT (MHproposal *MHp, Network *nwp)
   }
   
   BD_LOOP({
-      if(ndedges != 0 && unif_rand() < comp) { /* Select a discordant dyad at random */
+      if(ndedges != 0 && (nempty == 0 || unif_rand() < comp)) { /* Select a discordant dyad at random */
 	GetRandEdge(Mtail, Mhead, nwp+1);
 	
-	MHp->logratio += log(ndedges / ((double)nempty + 1)/comp * ((ndedges==1)? 1 : (1-comp)));
+	if(nempty==0){
+	  MHp->logratio += log(ndedges*(1-comp));
+	}else{
+	  if(ndedges==1){
+	    MHp->logratio += log(1.0 / (nempty + 1) /comp);
+	  }else{
+	    MHp->logratio += log((double) ndedges / (nempty + 1) / odds);
+	  }
+	}
       }else{ /* select an empty dyad in nwp[0] at random */
 	GetRandNonedge(Mtail, Mhead, nwp);
 	
 	if(ndedges==0){
 	  MHp->logratio += log(nempty*comp);
 	}else{
-	  MHp->logratio += log(((double)nempty)/(ndedges+1) *odds);
+	  if(nempty==1){
+	    MHp->logratio += log(1.0 / ndyads / (1-comp));
+	  }else{
+	    MHp->logratio += log((double) nempty / (ndedges+1) * odds);
+	  }
 	}
       }
     });
